<?php

/**
 * @file
 */

/**
 * Worldcore form theming function.
 */
function worldcore_payments_pre() {

  if (!empty($_POST['payments']) && isset($_POST['operation']) && ($_POST['operation'] == 'delete')) {
    return drupal_get_form('worldcore_payments_delete_confirm');
  }
  else {
    return drupal_get_form('worldcore_payments');
  }

}

/**
 * Worldcore payments listing form theming renderer function.
 */
function theme_worldcore_payments($form) {
  // Overview table:
  $header = array(
    theme('table_select_header_cell'),
    array('data' => t('ID'), 'field' => 'pid'),
    array('data' => t('Created'), 'field' => 'created'),
    array('data' => t('User'), 'field' => 'uid'),
    array('data' => t('Amount'), 'field' => 'amount'),
    array('data' => t('Currency'), 'field' => 'currency'),
    array('data' => t('Memo')),
    array('data' => t('WC track'), 'field' => 'track'),
    array('data' => t('Merchant account'), 'field' => 'merchant_account'),
    array('data' => t('Account'), 'field' => 'account'),
    array('data' => t('Enrolled'), 'field' => 'enrolled'),
  );

  $output = drupal_render($form['options']);
  if (isset($form['created']) && is_array($form['created'])) {
    foreach (element_children($form['created']) as $key) {
      $rows[] = array(
        drupal_render($form['payments'][$key]),
        drupal_render($form['pid'][$key]),
        drupal_render($form['created'][$key]),
        drupal_render($form['name'][$key]),
        drupal_render($form['amount'][$key]),
        drupal_render($form['currency'][$key]),
        drupal_render($form['memo'][$key]),
        drupal_render($form['track'][$key]),
        drupal_render($form['merchant_account'][$key]),
        drupal_render($form['account'][$key]),
        drupal_render($form['enrolled'][$key]),
      );
    }
  }
  else {
    $rows[] = array(array('data' => t('No payments available.'), 'colspan' => '11'));
  }

  $output .= theme('table', $header, $rows);
  if ($form['pager']['#value']) {
    $output .= drupal_render($form['pager']);
  }

  $output .= drupal_render($form);

  return $output;
}

/**
 * Worldcore payments listing form renderer function.
 */
function worldcore_payments(&$form_state) {

  $header = array(
    array(),
    array('data' => t('ID')),
    array('data' => t('ID'), 'field' => 'pid', 'sort' => 'asc'),
    array('data' => t('Created'), 'field' => 'created'),
    array('data' => t('User'), 'field' => 'uid'),
    array('data' => t('Amount'), 'field' => 'amount'),
    array('data' => t('Currency'), 'field' => 'currency'),
    array('data' => t('Memo')),
    array('data' => t('WC track'), 'field' => 'track'),
    array('data' => t('Merchant account'), 'field' => 'merchant_account'),
    array('data' => t('Account'), 'field' => 'account'),
    array('data' => t('Enrolled'), 'field' => 'enrolled'),
  );

  $currency_settings = unserialize(variable_get('worldcore_currencies', serialize(_worldcore_GetDefCurSetts())));

  $sql = 'SELECT * FROM {worldcore}';
  $sql .= tablesort_sql($header);
  $query_count = 'SELECT COUNT(pid) FROM {worldcore}';
  $result = pager_query($sql, 50, 0, $query_count);

  $form['options'] = array(
    '#type' => 'fieldset',
    '#title' => t('Update options'),
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  $form['options']['operation'] = array(
    '#type' => 'select',
    '#options' => array('delete' => t('Delete selected'), 'enroll' => t('Enroll selected')),
    '#default_value' => 'delete',
  );
  $form['options']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Apply'),
  );

  $payments = array();

  while ($payment = db_fetch_object($result)) {
    $payments[$payment->pid] = '';
    $form['pid'][$payment->pid] = array('#value' => $payment->pid);
    $form['created'][$payment->pid] = array('#value' => date("m/d/Y H:i", $payment->created));
    $form['name'][$payment->pid] = array('#value' => theme('username', user_load($payment->uid)));
    $form['amount'][$payment->pid] = array('#value' => round($payment->amount, $currency_settings[$payment->currency]['presc']));
    $form['currency'][$payment->pid] = array('#value' => $payment->currency);
    $form['memo'][$payment->pid] = array('#value' => $payment->memo);
    $form['track'][$payment->pid] = array('#value' => $payment->track);
    $form['merchant_account'][$payment->pid] = array('#value' => $payment->merchant_account);
    $form['account'][$payment->pid] = array('#value' => ((!empty($payment->account)) ? $payment->account : '-'));
    $form['enrolled'][$payment->pid] = array(
      '#value' => (($payment->enrolled > 0 && $payment->error == '') ? date("m/d/Y H:i", $payment->enrolled) : (!empty($payment->error) ? '<small><B>Error: </B>' . $payment->error . ' (' . date("m/d/Y H:i", $payment->enrolled) . ')</smal>' : '-')),
    );
  }

  $form['payments'] = array(
    '#type' => 'checkboxes',
    '#options' => $payments,
  );

  $form['pager'] = array('#value' => theme('pager', NULL, 50, 0));

  return $form;

}

/**
 * Worldcore payments form validation function.
 */
function worldcore_payments_validate($form, &$form_state) {
  $form_state['values']['payments'] = array_filter($form_state['values']['payments']);
  if (count($form_state['values']['payments']) == 0) {
    form_set_error('', t('No payments selected.'));
  }
}

/**
 * Worldcore payments form submit processing.
 */
function worldcore_payments_submit($form, &$form_state) {

  $payments = array_filter($form_state['values']['payments']);
  switch ($form_state['values']['operation']) {
    case 'enroll':
      $t = time();
      foreach ($payments as $pid) {
        if (_worldcore_enrollpayment($pid, 'MANUALLY', $t)) {
          module_invoke_all('worldcore', 'enrolled', $pid);
        }
      }
      drupal_set_message(t('The payments have been enrolled.'));
      break;
  }

}

/**
 * Worldcore delete payment form confirm function.
 */
function worldcore_payments_delete_confirm(&$form_state) {

  $edit = $form_state['post'];

  $form['payments'] = array('#tree' => TRUE);

  foreach (array_filter($edit['payments']) as $pid => $value) {

    $form['payments'][$pid] = array('#type' => 'hidden', '#value' => $pid);

  }

  $form['operation'] = array('#type' => 'hidden', '#value' => 'delete');

  return confirm_form($form,
                      t('Are you sure you want to delete selected payments?'),
                      'admin/worldcore/payments', t('This action cannot be undone.'),
                      t('Delete all selected'), t('Cancel'));
}

/**
 * Worldcore confirm delete form submit function.
 */
function worldcore_payments_delete_confirm_submit($form, &$form_state) {

  if ($form_state['values']['confirm']) {

    foreach ($form_state['values']['payments'] as $pid => $value) {
      _worldcore_deletepayment($pid);
    }

    drupal_set_message(t('The payments have been deleted.'));

  }

  $form_state['redirect'] = 'admin/worldcore/payments';

}

/**
 * Worldcore module settings form.
 */
function worldcore_settingsform() {

  $form = array();

  $form['API_key'] = array(
    '#type' => 'textfield',
    '#title' => t('API key'),
    '#default_value' => variable_get('worldcore_api_key', ''),
    '#size' => 40,
    '#maxlength' => 100,
    '#required' => FALSE,
  );

  $form['API_password'] = array(
    '#type' => 'textfield',
    '#title' => t('API password'),
    '#default_value' => variable_get('worldcore_api_password', ''),
    '#size' => 40,
    '#maxlength' => 100,
    '#required' => FALSE,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save changes'),
  );

  return $form;

}

/**
 *
 */
function worldcore_settingsform_validate($form, &$form_state) {

  // TODO.
}

/**
 * Worldcore currency settings form submit.
 */
function worldcore_settingsform_submit(&$form, $form_state) {

  variable_set('worldcore_api_key', $form_state['values']['API_key']);

  variable_set('worldcore_api_password', $form_state['values']['API_password']);

  drupal_set_message(t('Settings has been saved.'));
  drupal_goto('admin/worldcore/settings');

}

/**
 * Worldcore currency settings form.
 */
function worldcore_currencies() {

  $default_currency_settings = _worldcore_GetDefCurSetts();

  $currency_settings = unserialize(variable_get('worldcore_currencies', serialize($default_currency_settings)));

  foreach ($currency_settings as $key => $value) {

    $row = array();

    $row[] = array('data' => $key);
    $row[] = array('data' => ((empty($value['account'])) ? 'n/a' : $value['account']));
    $row[] = array('data' => (($value['enabled'] == 1) ? 'yes' : 'no'));
    $row[] = array(
      'data' => l(t('edit'), 'admin/worldcore/currencies/edit/' . $key) . ' ' . l(t('delete'), 'admin/worldcore/currencies/delete/' . $key) . ' ' . l(t('view sample'), 'admin/worldcore/sample/' . $key),
    );

    $rows[] = $row;

  }

  // Individual table headers.
  $header = array();
  $header[] = t('Currency');
  $header[] = t('WC account');
  $header[] = t('Enabled');
  $header[] = t('Action');

  $output = theme('table', $header, $rows);

  return $output;

}

/**
 * Worldcore currency edit form.
 */
function worldcore_currency_edit($form_state, $cur = '') {

  $currency_settings = unserialize(variable_get('worldcore_currencies', serialize(_worldcore_GetDefCurSetts())));

  $currency = $currency_settings[$cur];

  // Interface data:
  if (!empty($cur)) {

    $form['cur'] = array(
      '#type' => 'item',
      '#title' => t('Currency'),
      '#value' => $cur,
    );

    $form['currency'] = array(
      '#type' => 'hidden',
      '#value' => $cur,
    );

  }
  else {

    $form['currency'] = array(
      '#type' => 'textfield',
      '#title' => t('Currency'),
      '#description' => t('Enter currency ID string like "USD"'),
      '#default_value' => '',
      '#size' => 10,
      '#maxlength' => 15,
      '#required' => TRUE,
    );

    $currency['account'] = '';

    $currency['presc'] = '';

  }

  $form['account'] = array(
    '#type' => 'textfield',
    '#title' => t('WC account'),
    '#default_value' => $currency['account'],
    '#size' => 20,
    '#maxlength' => 15,
    '#required' => FALSE,
  );

  /*		$form['presc'] = array(
  '#type' => 'textfield',
  '#title' => t('Decimals'),
  '#default_value' => $currency['presc'],
  '#size' => 5,
  '#maxlength' => 3,
  '#required' => TRUE
		);*/

  $form['enabled'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enabled'),
    '#default_value' => $currency['enabled'],
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save changes'),
  );

  return $form;

}

/**
 * Worldcore currency edit form validation.
 */
function worldcore_currency_edit_validate($form, &$form_state) {

  if (empty($form_state['values']['currency'])) {
    form_set_error('', t('Currency ID does not specified.'));
  }

  if ($form_state['values']['enabled'] == '1') {
    if (empty($form_state['values']['account'])) {
      form_set_error('account', t('WC account can not be empty for enabled currency'));
    }
  }

  if (!empty($form_state['values']['account']) && !empty($form_state['values']['currency'])) {

    $letter = '';
    switch ($form_state['values']['currency']) {
      case 'USD':
        $letter = 'S';
        $currency = 'USD';
        break;

      case 'EUR':
        $letter = 'E';
        $currency = 'EUR';
        break;
    }

    if ($letter != '') {

      if ($letter != substr($form_state['values']['account'], 0, 1)) {
        form_set_error('account', t('WC account must start from !letter for !currency currency', array('!letter' => $letter, '!currency' => $currency)));
      }
    }

  }

}

/**
 * Worldcore currency edit form submit function.
 */
function worldcore_currency_edit_submit(&$form, &$form_state) {

  $currency_settings = unserialize(variable_get('worldcore_currencies', serialize(_worldcore_GetDefCurSetts())));

  $currency_settings["{$form_state['values']['currency']}"] = array(
    'enabled' => $form_state['values']['enabled'],
    'account' => $form_state['values']['account'],
  // 'presc' =>  (int)$form_state['values']['presc'].
    'presc' => 2,
  );

  variable_set('worldcore_currencies', serialize($currency_settings));

  drupal_set_message(t('Currency has been saved.'));

  $form_state['redirect'] = 'admin/worldcore/currencies';

}

/**
 * Worldcore currency delete form.
 */
function worldcore_currency_delete($form_state, $cur) {

  $form['cur'] = array(
    '#type' => 'value',
    '#value' => $cur,
  );

  return confirm_form($form,
    t('Are you sure you want to delete %cur ?', array('%cur' => $cur)),
    'admin/worldcore/currencies',
    t('This action cannot be undone.'),
    t('Delete'),
    t('Cancel')
  );
}

/**
 * Worldcore currency delete form submit function.
 */
function worldcore_currency_delete_submit($form, &$form_state) {

  if ($form_state['values']['confirm']) {

    $currency_settings = unserialize(variable_get('worldcore_currencies', serialize(_worldcore_GetDefCurSetts())));

    $out = array();
    foreach ($currency_settings as $key => $value) {
      if ($key != $form_state['values']['cur']) {
        $out[$key] = $value;
      }
    }

    if (count($out) < count($currency_settings)) {
      variable_set('worldcore_currencies', serialize($out));
      drupal_set_message(t('Currency has been deleted.'));
    }
    else {
      drupal_set_message(t('Currency has NOT been deleted.'));
    }

  }
  else {
    drupal_set_message(t('Currency has NOT been deleted.'));
  }

  $form_state['redirect'] = 'admin/worldcore/currencies';

}

/**
 * Sample settings page.
 */
function worldcore_sample() {

  global $base_path;

  return t('<h1>Setting up WorldCore</h1>
<p><A HREF="https://worldcore.eu/">Login</A> to your WC account, click on <A HREF="https://worldcore.eu/Customer/ApiSettings/Merchant">Settings</A> menu section and fill in the following fields:
<table cellpadding="5">
<tbody>
<tr>
	<td nowrap="nowrap">Status URL:</td>
	<td align="left"><input style="display: inline;" value="!status_url" size="50" id="m_name" name="m_name" type="text"></td>
</tr>
<tr>
	<td nowrap="nowrap">Success URL:</td>
	<td align="left"><input style="display: inline;" value="!success_url" size="50" id="m_name" name="m_name" type="text"></td>
</tr>
<tr>
	<td nowrap="nowrap">Failure URL:</td>
	<td align="left"><input style="display: inline;" value="!fail_url" size="50" id="m_name" name="m_name" type="text"></td>
</tr>
</tbody></table>',
  array(
    '!status_url' => 'https://' . $_SERVER['HTTP_HOST'] . $base_path . drupal_get_path_alias('worldcore/status_url'),
    '!success_url' => 'https://' . $_SERVER['HTTP_HOST'] . $base_path . drupal_get_path_alias('worldcore/success', ''),
    '!fail_url' => 'https://' . $_SERVER['HTTP_HOST'] . $base_path . drupal_get_path_alias('worldcore/fail', ''),
  ));
}
