<?php

/**
 * @file
 */

/**
 * Worldcore payment form.
 */
function worldcore_prefillform($form_state) {

  $form = array(); $currencies_ar = array();

  $currency_settings = unserialize(check_plain(variable_get('worldcore_currencies', serialize(_worldcore_GetDefCurSetts()))));
  foreach ($currency_settings as $key => $value) {
    if ($value['enabled'] == 1) {
      $currencies_ar[$key] = $key;
    }
  }

  $form['currency'] = array(
    '#type' => 'radios',
    '#title' => t("Currency"),
    '#options' => $currencies_ar,
    '#default_value' => 'USD',
        // '#description' => t("Select payment currency."),.
    '#required' => TRUE,
  );

  $form['amount'] = array(
    '#type' => 'textfield',
    '#title' => t('Amount'),
    '#default_value' => '',
    '#size' => 10,
    '#maxlength' => 12,
    '#required' => TRUE,
  );

  $form['memo'] = array(
    '#type' => 'textarea',
    '#title' => t('Memo'),
    '#default_value' => t('Payment to !sitename', array('!sitename' => variable_get('site_name', 'Drupal'))),
    '#description' => t('Payment description.'),
    '#required' => TRUE,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Cerate payment'),
  );

  return $form;

}

/**
 * Worldcore payment form submit function.
 */
function worldcore_prefillform_submit(&$form, $form_state) {

  $payment = _worldcore_createpayment(array(
    'amount' => $form_state['values']['amount'],
    'currency' => $form_state['values']['currency'],
    'memo' => $form_state['values']['memo'],
  ));

  if (is_array($payment) && $payment['pid'] > 0) {
    drupal_set_message(t('Please confirm payment details'));
    drupal_goto('worldcore/payment/' . $payment['pid']);
  }

}

/**
 * Worldcore redirect from.
 */
function worldcore_sciform($form_state, $payment) {

  // Fetch payment info from DB.
  if (!is_array($payment) && is_integer($payment)) {
    $payment = worldcore_pid_load($payment);
  }

  if (!is_array($payment) || !($payment['pid'] > 0)) {

    $form['error'] = array(
      '#type' => 'item',
      '#title' => t('Error'),
      '#value' => t('Order you are going to pay for does not exist'),
    );

  }
  else {

    $currency_settings = unserialize(check_plain(variable_get('worldcore_currencies', serialize(_worldcore_GetDefCurSetts()))));
    $presc = $currency_settings["{$payment['currency']}"]['presc'];

    $post_str = json_encode(array(
      'account' => $payment['merchant_account'],
      'amount' => round($payment['amount'], $presc),
      'invoiceId' => $payment['pid'],
      'customField' => $payment['memo'],
    ));
    $hash_in = strtoupper(hash('sha256', $post_str . variable_get('worldcore_api_password', '')));
    $auth_header = 'Authorization: wauth key=' . variable_get('worldcore_api_key', '') . ', hash=' . $hash_in;
    $curl = curl_init();
    curl_setopt($curl, CURLOPT_URL, 'https://api.worldcore.eu/v1/merchant');
    curl_setopt($curl, CURLOPT_HEADER, TRUE);
    curl_setopt($curl, CURLOPT_POST, 1);
    curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type: application/json; charset=utf-8', $auth_header));
    curl_setopt($curl, CURLOPT_POSTFIELDS, $post_str);
    curl_setopt($curl, CURLOPT_TIMEOUT, 10);
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, TRUE);
    $curl_response = curl_exec($curl);
    if ($curl_response == FALSE) {

      $error_msg = curl_error($curl);

      watchdog('WorldCore', 'CURL error: ' . $error_msg, array(), WATCHDOG_INFO);

      return;

    }
    else {
      list($response_headers, $json_response) = explode("\r\n\r\n", $curl_response, 2);
      preg_match("/^WSignature: ([A-Z0-9]{64})\r$/m", $response_headers, $hash_outputed);
      $hash_check = strtoupper(hash('sha256', $json_response . variable_get('worldcore_api_password', '')));
      if ($hash_outputed[1] != $hash_check) {

        watchdog('WorldCore', 'Hash not match!', array(), WATCHDOG_INFO);

        return;

      }
      else {
        $decoded_response = json_decode($json_response, TRUE);
        if (isset($decoded_response['error'])) {

          watchdog('WorldCore', 'Error occurred: ' . print_r($decoded_response['error'], TRUE), array(), WATCHDOG_INFO);

          return;

        }
      }
    }
    curl_close($curl);

    // Interface data:
    $form['payment_id'] = array(
      '#type' => 'item',
      '#title' => t('Order #'),
      '#value' => $payment['pid'],
    );

    $form['amount'] = array(
      '#type' => 'item',
      '#title' => t('Amount'),
      '#value' => round($payment['amount'], $presc) . ' ' . $payment['currency'],
    );

    $form['memo'] = array(
      '#type' => 'item',
      '#title' => t('Memo'),
      '#value' => $payment['memo'],
    );

    // SCI data...
    $form['#action'] = $decoded_response['data']['url'];

    $form['#method'] = 'get';

    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Pay now'),
    );

  }

  return $form;

}

/**
 * Worldcore payment success page.
 */
function worldcore_success() {

  return theme('worldcore_success');

}

/**
 * Worldcore payment failed page.
 */
function worldcore_fail() {
  return theme('worldcore_fail');
}

/**
 * Worldcore status_url processing function.
 */
function worldcore_status() {

  drupal_set_header('Content-type: text/html; charset=UTF-8');

  // Check url.
  $url = trim($_SERVER['REQUEST_URI'], '/');
  $alias = drupal_get_path_alias('worldcore/status_url');
  if ($url != $alias) {
    die();
  }

  $headers = apache_request_headers();
  $json_body = file_get_contents('php://input');
  $hash_check = strtoupper(hash('sha256', $json_body . variable_get('worldcore_api_password', '')));
  // Hash checking.
  if ($headers['WSignature'] == $hash_check) {

    // Some code to confirm payment.
    $created = time();

    $decoded_response = json_decode($json_body, TRUE);

    $payment = worldcore_pid_load($decoded_response['invoiceId']);

    // Enroll payment.
    db_query('UPDATE {worldcore} SET track=\'%s\', account=\'%s\', enrolled=%d WHERE pid=%d', $decoded_response['track'], $decoded_response['account'], $created, $payment['pid']);

    // Fire hook.
    $payment['track'] = $decoded_response['track'];
    $payment['account'] = $decoded_response['account'];
    $payment['enrolled'] = $created;
    module_invoke_all('worldcore', 'enrolled', $payment['pid'], $payment);

  }
  else {

    // Some code to log invalid payments.
    watchdog('WorldCore', 'Hash mismatch! ' . $headers['WSignature'] . ' vs. ' . $hash_check, array(), WATCHDOG_ERROR);

    die();

  }

}
